@@ -45,7 +45,7 @@
     (and (symbolp x) (remprop x 'opr))
     (and (stringp x) (remhash x *opr-table*))))
 
-;; Store build-in operators, which get additional properties.
+;; Store built-in operators, which get additional properties.
 ;; These operators aren't killed by the function kill-operator.
 (defvar *mopl* nil)
 
@@ -61,16 +61,16 @@
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'op))
       '((mqapply $subvar) (bigfloat $bfloat)))
 
-(setq $exptsubst nil
-      $partswitch nil
-      $inflag nil
-      $gradefs '((mlist simp))
-      $dependencies '((mlist simp))
-      atvars '($@1 $@2 $@3 $@4)
-      $derivsubst nil
-      $opsubst t
-      in-p nil
-      substp nil)
+(defmvar $exptsubst nil)
+(defmvar $partswitch nil)
+(defmvar $inflag nil)
+(defmvar $derivsubst nil)
+(defmvar $opsubst t)
+(defvar $gradefs '((mlist simp)))
+(defvar $dependencies '((mlist simp)))
+(defvar atvars '($@1 $@2 $@3 $@4))
+(defvar in-p nil)
+(defvar substp nil)
 
 (defmvar $vect_cross nil
   "If TRUE allows DIFF(X~Y,T) to work where ~ is defined in
@@ -197,8 +197,7 @@
   (let (newexpt)
     (cond ((atom z) z)
 	  ((specrepp z) (subst2 x y (specdisrep z) negxpty timesp))
-	  ((and *atp* (member (caar z) '(%derivative %laplace) :test #'eq)) z)
-	  ((at-substp z) z)
+	  ((at-substp z) z) ;; IS SUBST-EXCEPT-SECOND-ARG APPROPRIATE HERE ?? !!
 	  ((alike1 y z) x)
 	  ((and timesp (eq (caar z) 'mtimes) (alike1 y (setq z (nformat z)))) x)
 	  ((and (eq (caar y) 'mexpt) (eq (caar z) 'mexpt) (alike1 (cadr y) (cadr z))
@@ -218,15 +217,39 @@
 ;; and we do not wish to subst 3 for x inside integrand.
 (defun subst-except-second-arg (x y z)
   (cond 
-    ((member (caar z) '(%integrate %sum %product %limit))
+    ((member (caar z) '(%integrate %sum %product %limit %laplace))
      (append 
-       (list (car z)
+       (list (remove 'simp (car z))   ; ensure resimplification after substitution
              (if (eq y (third z))     ; if (third z) is new var that shadows y
                  (second z)           ; leave (second z) unchanged
                  (subst1 x y (second z))) ; otherwise replace y with x in (second z)
              (third z))               ; never change integration var
        (mapcar (lambda (z) (subst1 x y z)); do subst in limits of integral
                (cdddr z))))
+    ((eq (caar z) '%at)
+     ;; similar considerations here, but different structure of expression.
+     (let*
+       ((at-eqn-or-eqns (third z))
+        (at-eqns-list (if (eq (caar at-eqn-or-eqns) 'mlist) (rest at-eqn-or-eqns) (list at-eqn-or-eqns))))
+       (list
+         (remove 'simp (car z)) ;; ensure resimplification after substitution
+         (if (member y (mapcar #'(lambda (e) (second e)) at-eqns-list))
+           (second z)
+           (subst1 x y (second z)))
+         `((mlist) ,@(mapcar #'(lambda (e) (list (first e) (second e) (subst1 x y (third e)))) at-eqns-list)))))
+    ((eq (caar z) '%derivative)
+     ;; and again, with yet a different structure.
+     (let*
+       ((vars-and-degrees (rest (rest z)))
+        (diff-vars (loop for v in vars-and-degrees by #'cddr collect v))
+        (diff-degrees (loop for n in (rest vars-and-degrees) by #'cddr collect n)))
+       (append
+         (list
+           (remove 'simp (car z)) ;; ensure resimplification after substitution
+           (if (member y diff-vars)
+             (second z)
+             (subst1 x y (second z))))
+         (apply #'append (loop for v in diff-vars for n in diff-degrees collect (list v (subst1 x y n)))))))
     (t z)))
 
 (defmfun subst0 (new old)
@@ -393,7 +416,7 @@
 		(t (go noun)))
      doit (cond ((nonvarcheck (cadr z) '$diff))
 		((null (cddr z)) (wna-err '$diff))
-		((not (eq (ml-typep (caddr z)) 'fixnum)) (go noun))
+		((not (fixnump (caddr z))) (go noun))
 		((minusp (setq count (caddr z)))
 		 (merror (intl:gettext "diff: order of derivative must be a nonnegative integer; found ~M") count)))
      loop1(cond ((zerop count) (rplacd z (cdddr z)) (go loop2))
@@ -513,7 +536,7 @@
 
 (defun sdiffgrad (e x)
   (let ((fun (caar e)) grad args result)
-    (cond ((and (eq fun 'mqapply) (oldget (caaadr e) 'grad))
+    (cond ((and (eq fun 'mqapply) (zl-get (caaadr e) 'grad))
            ;; Change the array function f[n](x) to f(n,x), call sdiffgrad again.
            (setq result
 	         (sdiffgrad (cons (cons (caaadr e) nil) 
@@ -527,13 +550,13 @@
                result))
 
 	  ;; extension for pdiff.
-	  ((and (get '$pderivop 'operators) (sdiffgrad-pdiff e x)))
+	  ((and (get '$pderivop 'operators) (funcall 'sdiffgrad-pdiff e x)))
 
 	  ;; two line extension for hypergeometric.
 	  ((and (equal fun '$hypergeometric) (get '$hypergeometric 'operators))
-	   (diff-hypergeometric (second e) (third e) (fourth e) x))
+	   (funcall 'diff-hypergeometric (second e) (third e) (fourth e) x))
 
-	  ((or (eq fun 'mqapply) (null (setq grad (oldget fun 'grad))))
+	  ((or (eq fun 'mqapply) (null (setq grad (zl-get fun 'grad))))
 	   (if (not (depends e x)) 0 (diff%deriv (list e x 1))))
 	  ((not (= (length (cdr e)) (length (car grad))))
 	   (merror (intl:gettext "~:M: expected exactly ~M arguments.") 
@@ -716,6 +739,7 @@
 (defmfun nonvarcheck (e fn)
   (if (or (mnump e)
 	  (maxima-integerp e)
+	  ($constantp e)
 	  (and (not (atom e)) (not (eq (caar e) 'mqapply)) (mopp1 (caar e))))
       (merror (intl:gettext "~:M: second argument must be a variable; found ~M") fn e)))
 
@@ -1096,6 +1120,24 @@
     (merror (intl:gettext "last: empty argument.")))
   (car (last e)))
 
+(defmfun $firstn (e n)
+  (atomchk (setq e (format1 e)) '$firstn nil)
+  (if (or (not (fixnump n)) (minusp n))
+    (merror (intl:gettext "firstn: second argument must be a nonnegative integer; found: ~M") n))
+  (let ((m ($length e)))
+    (if (> n m)
+      ($rest e 0)
+      ($rest e (- n m)))))
+
+(defmfun $lastn (e n)
+  (atomchk (setq e (format1 e)) '$lastn nil)
+  (if (or (not (fixnump n)) (minusp n))
+    (merror (intl:gettext "lastn: second argument must be a nonnegative integer; found: ~M") n))
+  (let ((m ($length e)))
+    (if (> n m)
+      ($rest e 0)
+      ($rest e (- m n)))))
+
 (defmfun $args (e)
   (atomchk (setq e (format1 e)) '$args nil)
 	 (cons '(mlist) (margs e)))
@@ -1250,6 +1292,8 @@
 	 ;; Handle like erf(%i).  float(%i) (via recur-apply below)
 	 ;; just returns %i, so we never numerically evaluate it.
 	 (complexify (complex-erf (complex 0 1d0))))
+	((or (eq (caar e) '%derivative) (eq (caar e) '$diff))
+	 (append (list (remove 'simp (first e)) ($float (second e))) (rest (rest e))))
 	(t (recur-apply #'$float e))))
 
 (defmfun $coeff (e x &optional (n 1))
