@@ -218,7 +218,7 @@
                                   (cons (/ i (1- n)) map))))))    ;; number i
 
            ;; prints map with the format:  nj, "cj", ...,n1, "c1"  
-           (setq fun (format nil "~{~f ~s~^, ~}" (reverse map)))
+           (setq fun (format nil "~{~,8f ~s~^, ~}" (reverse map)))
            ;; outputs the string: defined (nj, "cj", ...,n1, "c1")
            (format st "defined (~a)" fun)))
         (t
@@ -232,6 +232,9 @@
         (meshcolor (if (member :mesh_lines_color plot-options)
                        (getf plot-options :mesh_lines_color)
                        '$black)))
+    (when (and (member :gnuplot_pm3d plot-options)
+               (not (getf plot-options :gnuplot_pm3d)))
+      (setq palette nil))
     (when (find 'mlist palette :key #'car) (setq palette (list palette)))
     ;; user's preamble
     (when (and (getf plot-options :gnuplot_preamble)
@@ -241,6 +244,13 @@
     ;; sets-up terminal command and output file name
     (setq terminal-file (gnuplot-terminal-and-file plot-options))
 
+    ;; By default gnuplot assumes everything below 1e-8 to be a rounding error
+    ;; and rounds it down to 0. This is handy for standalone gnuplot as it allows
+    ;; to suppress pixels with imaginary part while allowing for small calculation
+    ;; errors. As plot and draw handle the imaginary part without gnuplot's help
+    ;; this isn't needed here and is turned off as it often surprises users.
+    (format dest "set zero 0.0~%")
+
     ;; prints terminal and output commands
     (when (first terminal-file)
       (format dest "~a~%" (first terminal-file)))
@@ -305,7 +315,7 @@
     (when (getf plot-options :zlabel)
       (format dest "set zlabel ~s~%" (getf plot-options :zlabel)))
     (when (and (member :legend plot-options) 
-               (not (first (getf plot-options :legend))))
+               (null (getf plot-options :legend)))
       (format dest "unset key~%"))
 
     ;; plotting box
@@ -328,11 +338,11 @@
       (if (getf plot-options :same_xy)
           (format dest "set size ratio -1~%")
           (if (getf plot-options :yx_ratio)
-              (format dest "set size ratio ~f~%" (getf plot-options :yx_ratio))
+              (format dest "set size ratio ~,8f~%" (getf plot-options :yx_ratio))
               (format dest "set size ratio 0.75~%")))
       (if (and (getf plot-options :xy_scale)
                (listp (getf plot-options :xy_scale)))
-          (format dest "set size ~{~f~^, ~}~%" (getf plot-options :xy_scale))))
+          (format dest "set size ~{~,8f~^, ~}~%" (getf plot-options :xy_scale))))
 
     ;; plot size and aspect ratio for plot3d
     (when (string= (getf plot-options :type) "plot3d")
@@ -341,45 +351,45 @@
       (when (getf plot-options :same_xyz)
         (format dest "set view equal xyz~%"))
       (when (getf plot-options :zmin)
-        (format dest "set xyplane at ~f~%" (getf plot-options :zmin))))
+        (format dest "set xyplane at ~,8f~%" (getf plot-options :zmin))))
 
     ;; axes tics
     (when (member :xtics plot-options)
       (let ((xtics (getf plot-options :xtics)))
         (if (consp xtics)
-            (format dest "set xtics ~{~f~^, ~}~%" xtics)
+            (format dest "set xtics ~{~,8f~^, ~}~%" xtics)
             (if xtics
-                (format dest "set xtics ~f~%" xtics)
+                (format dest "set xtics ~,8f~%" xtics)
                 (format dest "unset xtics~%")))))
     (when (member :ytics plot-options)
       (let ((ytics (getf plot-options :ytics)))
         (if (consp ytics)
-            (format dest "set ytics ~{~f~^, ~}~%" ytics)
+            (format dest "set ytics ~{~,8f~^, ~}~%" ytics)
             (if ytics
-                (format dest "set ytics ~f~%" ytics)
+                (format dest "set ytics ~,8f~%" ytics)
                 (format dest "unset ytics~%")))))
     (when (member :ztics plot-options)
       (let ((ztics (getf plot-options :ztics)))
         (if (consp ztics)
-            (format dest "set ztics ~{~f~^, ~}~%" ztics)
+            (format dest "set ztics ~{~,8f~^, ~}~%" ztics)
             (if ztics
-                (format dest "set ztics ~f~%" ztics)
+                (format dest "set ztics ~,8f~%" ztics)
                 (format dest "unset ztics~%")))))
     (when (member :color_bar_tics plot-options)
       (let ((cbtics (getf plot-options :color_bar_tics)))
         (if (consp cbtics)
-            (format dest "set cbtics ~{~f~^, ~}~%" cbtics)
+            (format dest "set cbtics ~{~,8f~^, ~}~%" cbtics)
             (if cbtics
-                (format dest "set cbtics ~f~%" cbtics)
+                (format dest "set cbtics ~,8f~%" cbtics)
                 (format dest "unset cbtics~%")))))
 
     ;; axes ranges and style
     (when (and (getf plot-options :x) (listp (getf plot-options :x)))
-      (format dest "set xrange [~{~f~^ : ~}]~%" (getf plot-options :x)))
+      (format dest "set xrange [~{~,8f~^ : ~}]~%" (getf plot-options :x)))
     (when (and (getf plot-options :y) (listp (getf plot-options :y)))
-      (format dest "set yrange [~{~f~^ : ~}]~%" (getf plot-options :y)))
+      (format dest "set yrange [~{~,8f~^ : ~}]~%" (getf plot-options :y)))
     (when (and (getf plot-options :z) (listp (getf plot-options :z)))
-      (format dest "set zrange [~{~f~^ : ~}]~%" (getf plot-options :z)))
+      (format dest "set zrange [~{~,8f~^ : ~}]~%" (getf plot-options :z)))
     (when (and (string= (getf plot-options :type) "plot2d")
                (member :axes plot-options))
       (if (getf plot-options :axes)
@@ -395,7 +405,7 @@
     (when (getf plot-options :label)
       (dolist (label (getf plot-options :label))
         (when (and (listp label) (= (length label) 4))
-          (format dest "set label ~s at ~{~f~^, ~}~%"
+          (format dest "set label ~s at ~{~,8f~^, ~}~%"
                   (cadr label) (cddr label)))))
 
     ;; identifier for missing data
@@ -406,8 +416,8 @@
                (> (length  (getf plot-options :gnuplot_postamble)) 0))
       (format dest "~a~%" (getf plot-options :gnuplot_postamble)))
 
-    ;;returns the name of the file created
-    (or (second terminal-file) "")))
+    ;;returns a list with the name of the file created, or nil
+    (if (null (second terminal-file)) nil (list (second terminal-file)))))
 
 (defun gnuplot-plot3d-command (file palette gstyles colors titles n) 
 (let (title (style "with pm3d"))
@@ -473,7 +483,7 @@
          (setq terminal-command
                (getf plot-options :gnuplot_default_term_command))
          (setq terminal-command
-               "set term wxt size 640,480 font \",12\"; set term pop")))
+               "set term pop")))
     ((getf plot-options :gnuplot_term)
      (setq
       terminal-command
