@@ -113,8 +113,9 @@
 		      (go a1))
 		     (t
 		      ;; Almost nobody knows what this means. Just suppress the noise.
-		      ;; (mtell "COMPILEPLUS: ~M partitions '+' expression.~%" (cons '(mplus) leftover))
-		      (setq boundlist (append boundlist (atomson leftover)))
+                      ;; (mtell "COMPILEPLUS: ~M partitions '+'
+                      ;; expression.~%" (cons '(mplus) leftover))
+		      (setq boundlist (append boundlist (remove-if-not #'atom leftover)))
 		      (return (emit (list 'cond
 					  (list (list 'part+
 						      e
@@ -138,20 +139,20 @@
 	       (cond ((null (cdr p)) (return nil)) (t (go a))))
 	      ((eq (caaar p) 'mtimes)
 	       (cond ((and (not (or (numberp (cadar p))
-				    (and (not (atom (cadar p)))
-					 (eq (caar (cadar p)) 'rat))))
-			   (fixedmatchp (cadar p)))
+                               (and (not (atom (cadar p)))
+                                  (eq (caar (cadar p)) 'rat))))
+                         (fixedmatchp (cadar p)))
 		      (setq flag nil)
 		      (emit `(setq ,(genref)
-			      (ratdisrep
-			       (ratcoef ,e ,(memqargs (cadar p))))))
+                                   (ratdisrep
+                                    (ratcoef ,e ,(memqargs (cadar p))))))
 		      (compiletimes (car reflist) (cons '(mtimes) (cddar p)))
 		      (emit `(setq ,e (meval
 				       (quote
 					(($ratsimp)
 					 ((mplus) ,e
-					  ((mtimes) -1 ,(car reflist)
-					   ,(cadar p)))))))))
+                                                  ((mtimes) -1 ,(car reflist)
+                                                            ,(cadar p)))))))))
 		     ((null flag)
 		      (setq flag t) (rplacd (car p) (reverse (cdar p))) (go a1))
 		     (t (setq leftover (cons (car p) leftover)) (go a))))
@@ -188,29 +189,29 @@
 		      (setq leftover (cons (car p) leftover))
 		      (setq p (cdr p))
 		      (go a1))
-                      (leftover (setq leftover (cons (car p) leftover)) (setq p nil) (go a1)))
+                     (leftover (setq leftover (cons (car p) leftover)) (setq p nil) (go a1)))
 	       (setq boundlist (cons (caaar p) boundlist))
 	       (emit (list 'msetq
 			   (caaar p)
 			   (list 'kaar e)))
 	       (go functionmatch))
 	      (t (go functionmatch)))
-   (go a)
+     (go a)
    functionmatch
-   (emit (list 'setq
-	       (genref)
-	       (list 'findfun e (memqargs (caaar p)) ''mplus)))
-   (cond ((eq (caaar p) 'mplus)
-	  (mtell (intl:gettext "COMPILEPLUS: warning: '+' within '+' in: ~M~%") (car p))
-	  (compileplus (car reflist) (car p)))
-	 (t (emit (list 'setq (genref) (list 'kdr (cadr reflist))))
-	    (compileeach (car reflist) (cdar p))))
-   (emit (list 'setq
-	       e
-	       (list 'meval
-		     (list 'quote
-			   (list '(mplus) e (list '(mminus) (car p)))))))
-   (go a)))
+     (emit (list 'setq
+                 (genref)
+                 (list 'findfun e (memqargs (caaar p)) ''mplus)))
+     (cond ((eq (caaar p) 'mplus)
+            (mtell (intl:gettext "COMPILEPLUS: warning: '+' within '+' in: ~M~%") (car p))
+            (compileplus (car reflist) (car p)))
+           (t (emit (list 'setq (genref) (list 'kdr (cadr reflist))))
+              (compileeach (car reflist) (cdar p))))
+     (emit (list 'setq
+                 e
+                 (list 'meval
+                       (list 'quote
+                             (list '(mplus) e (list '(mminus) (car p)))))))
+     (go a)))
 
 (defun compiletimes (e p) 
   (prog (reflist f g h leftover) 
@@ -232,7 +233,7 @@
 		     (t
 		      ;; Almost nobody knows what this means. Just suppress the noise.
 		      ;; (mtell "COMPILETIMES: ~M partitions '*' expression.~%" (cons '(mtimes) leftover))
-		      (setq boundlist (append boundlist (atomson leftover)))
+		      (setq boundlist (append boundlist (remove-if-not #'atom leftover)))
 		      (return (emit (list 'cond
 					  (list (list 'part*
 						      e
@@ -292,21 +293,21 @@
 			   (list 'kaar e)))
 	       (go functionmatch))
 	      (t (go functionmatch)))
-   (go a)
+     (go a)
    functionmatch
-   (emit (list 'setq
-	       (genref)
-	       (list 'findfun e (memqargs (caaar p)) ''mtimes)))
-   (cond ((eq (caaar p) 'mtimes)
-	  (mtell (intl:gettext "COMPILETIMES: warning: '*' within '*' in: ~M~%") (car p))
-	  (compiletimes (car reflist) (car p)))
-	 (t (emit (list 'setq (genref) (list 'kdr (cadr reflist))))
-	    (compileeach (car reflist) (cdar p))))
-   (emit (list 'setq
-	       e
-	       (list 'meval
-		     (list 'quote (list '(mquotient) e (car p))))))
-   (go a)))
+     (emit (list 'setq
+                 (genref)
+                 (list 'findfun e (memqargs (caaar p)) ''mtimes)))
+     (cond ((eq (caaar p) 'mtimes)
+            (mtell (intl:gettext "COMPILETIMES: warning: '*' within '*' in: ~M~%") (car p))
+            (compiletimes (car reflist) (car p)))
+           (t (emit (list 'setq (genref) (list 'kdr (cadr reflist))))
+              (compileeach (car reflist) (cdar p))))
+     (emit (list 'setq
+                 e
+                 (list 'meval
+                       (list 'quote (list '(mquotient) e (car p))))))
+     (go a)))
 
 
 (defmspec $defmatch (form)
@@ -344,13 +345,6 @@
 	      (meta-mputprop name (list '(mlist) pt* (cons '(mlist) args)) '$rule)
 	      (return name)))))
 
-
-(defun atomson (l) 
-  (cond ((null l) nil)
-	((atom (car l)) (cons (car l) (atomson (cdr l))))
-	(t (atomson (cdr l)))))
-
-
 (defmspec $tellsimp (form)
   (let ((meta-prop-p nil))
     (proc-$tellsimp (cdr form))))
