@@ -115,7 +115,7 @@
 
 (defun lnewvar1 (a)
   (cond ((atom a) (newvar1 a))
-	((member (caar a) '(mlist mequal $matrix) :test #'eq) (mapc #'lnewvar1 (cdr a)))
+	((mbagp a) (mapc #'lnewvar1 (cdr a)))
 	(t (newvar1 a))))
 
 (defun newvarmat (mat1 mat2)
@@ -240,8 +240,8 @@
 
 (defun diagmatrix (n var fn)
   (prog (i ans)
-     (if (or (not (eq (ml-typep n) 'fixnum)) (minusp n))
-	 (improper-arg-err n fn))
+     (when (or (not (fixnump n)) (minusp n))
+       (improper-arg-err n fn))
      (setq i n)
      loop (if (zerop i) (return ans))
      (setq ans (cons (onen i n var 0) ans) i (1- i))
@@ -269,7 +269,7 @@
      (go loop2)))
 
 (defmfun $invert_by_gausselim (k) 
-  (let ((*inv* t) *det* linind* top* mul* ($ratmx t) (ratmx $ratmx) $ratfac $sparse)
+  (let ((*inv* t) *det* top* mul* ($ratmx t) (ratmx $ratmx) $ratfac $sparse)
     (cond ((atom k) ($nounify '$inverx) (list '(%inverx) k))
 	  (t (newvarmat1 (setq k (check k)))
 	     (setq k (invert1 (replist1 (mcx (cdr k)))))
@@ -299,7 +299,7 @@
 		 (t (ratinvert elm))))))
 
 (defun invert1 (k) 
-  (prog (l r g i m n ei* ej* oneoff*) 
+  (prog (l r g i m n)
      (setq l (length k) i 1) 
      (cond ((= l (length (car k))) nil)
 	   (t(merror (intl:gettext "invert: matrix must be square; found ~M rows, ~M columns.") l (length (car k)))))
@@ -585,7 +585,9 @@
 (defmfun $triangularize (x) 
   (let (($ratmx t))
     (newvarmat1 (setq x (check x))))
-  (setq x (cons '($matrix) (mxc (disreplist1 (triang (replist1 (mcx (cdr x)))))))) 
+  (let (($algebraic $algebraic))
+    (and (not $algebraic) (some #'algp varlist) (setq $algebraic t))
+    (setq x (cons '($matrix) (mxc (disreplist1 (triang (replist1 (mcx (cdr x)))))))))
   (if $ratmx x ($totaldisrep x)))
 
 (defmfun $col (mat n)
