@@ -61,6 +61,9 @@ call."
 	  ((not (eq name (caar fun)))	;efficiency hack I guess
 	   (rplaca (car fun) name)))	;  done in jpg;mlisp
     (setq args (cdr fun))		;  (in MDEFINE).
+    (let ((dup (find-duplicate args :test #'eq :key #'mparam)))
+      (when dup
+        (merror (intl:gettext "macro definition: ~M occurs more than once in the parameter list") (mparam dup))))
     (mredef-check name)
     (do ((a args (cdr a)) (mlexprp))
 	((null a)
