@@ -2,6 +2,11 @@
 
 (in-package :maxima)
 
+(defvar *debug-quadpack*
+  nil
+  "Set to non-NIL to enable printing of the error object when the
+  Slatec routines throw an error.")
+
 #-(or gcl ecl)
 (defmacro get-integrand (fun var)
   `(compile nil (coerce-float-fun ,fun `((mlist) ,,var))))
@@ -16,6 +21,21 @@
 	v
 	(error (intl:gettext "~A is not a real floating-point number") val))))
 
+;; Convert the maxima function FUN of the one variable VAR to a
+;; compiled lisp function returning a float that is suitable for use
+;; as the integrand for the quadpack routines.  If the integrand
+;; cannot be evaluated at the specified point, signal an error.
+(defmacro float-integrand-or-lose (name fun var)
+  (let ((f (gensym "COMPILED-FUN-"))
+	(x (gensym "VAR-"))
+	(result (gensym "RESULT-")))
+    `(let ((,f (get-integrand ,fun ,var)))
+       (lambda (,x)
+	 (let ((,result (funcall ,f ,x)))
+	   (unless (cl:numberp ,result)
+	     (merror (intl:gettext "~M: Cannot numerically evaluate ~M at ~M") ,name ,fun ,x))
+	   (float ,result))))))
+     
 (defun quad-qag (fun var a b key &key
 		 (epsrel 1e-8)
 		 (limit 200)
@@ -23,13 +43,11 @@
   (quad_argument_check fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
-	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var)))
+	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
     (handler-case
 	(multiple-value-bind (junk z-a z-b z-epsabs z-epsrel z-key result abserr neval ier
 				   z-limit z-lenw last)
-	    (slatec:dqag #'(lambda (x)
-			     (float (funcall f x)))
+	    (slatec:dqag (float-integrand-or-lose '$quad_qag fun var)
 			 (float-or-lose a)
 			 (float-or-lose b)
 			 (float-or-lose epsabs)
@@ -39,7 +57,9 @@
 			 limit lenw 0 iwork work)
 	  (declare (ignore junk z-a z-b z-epsabs z-epsrel z-key z-limit z-lenw last))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qag) ,fun ,var ,a ,b ,key
 	  ((mequal) $epsrel ,epsrel)
 	  ((mequal) $epsabs ,epsabs)
@@ -52,13 +72,11 @@
   (quad_argument_check fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
-	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var)))
+	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
     (handler-case
 	(multiple-value-bind (junk z-a z-b z-epsabs z-epsrel result abserr neval ier
 				   z-limit z-lenw last)
-	    (slatec:dqags #'(lambda (x)
-			      (float (funcall f x)))
+	    (slatec:dqags (float-integrand-or-lose '$quad_qags fun var)
 			  (float-or-lose a)
 			  (float-or-lose b)
 			  (float-or-lose epsabs)
@@ -67,7 +85,9 @@
 			  limit lenw 0 iwork work)
 	  (declare (ignore junk z-a z-b z-epsabs z-epsrel z-limit z-lenw last))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qags) ,fun ,var ,a ,b
 	  ((mequal) $epsrel ,epsrel)
 	  ((mequal) $epsabs ,epsabs)
@@ -94,11 +114,7 @@
 		    (setq bnd ($float low))
 		    (setq inf high))
 		   (t
-		    (return-from quad-qagi
-		      `(($quad_qagi) ,fun ,var ,a ,b
-			((mequal) $epsrel ,epsrel)
-			((mequal) $epsabs ,epsabs)
-			((mequal) $limit ,limit)))))
+		    (merror "~M: Unexpected limits of integration: ~M, ~M~%" '$quad_qagi low high)))
 	     (values bnd inf))))
 
     (multiple-value-bind (bound inf-type)
@@ -106,7 +122,6 @@
       (let* ((lenw (* 4 limit))
 	     (work (make-array lenw :element-type 'flonum))
 	     (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	     (f (get-integrand fun var))
 	     (infinity (ecase inf-type
 			 ((1 $inf)
 			  ;; Interval is [bound, infinity]
@@ -121,8 +136,7 @@
 	    (multiple-value-bind (junk z-bound z-inf z-epsabs z-epsrel
 				       result abserr neval ier
 				       z-limit z-lenw last)
-		(slatec:dqagi #'(lambda (x)
-				  (float (funcall f x)))
+		(slatec:dqagi (float-integrand-or-lose '$quad_qagi fun var)
 			      (float-or-lose bound)
 			      infinity
 			      (float-or-lose epsabs)
@@ -132,7 +146,9 @@
 	      (declare (ignore junk z-bound z-inf z-epsabs z-epsrel
 			       z-limit z-lenw last))
 	      (list '(mlist) result abserr neval ier))
-	  (error ()
+	  (error (e)
+	    (when *debug-quadpack*
+	      (format t "~S" e))
 	    `(($quad_qagi) ,fun ,var ,a ,b
 	      ((mequal) $epsrel ,epsrel)
 	      ((mequal) $epsabs ,epsabs)
@@ -145,13 +161,11 @@
   (quad_argument_check fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
-	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var)))
+	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
     (handler-case
 	(multiple-value-bind (junk z-a z-b z-c z-epsabs z-epsrel result abserr neval ier
 				   z-limit z-lenw last)
-	    (slatec:dqawc #'(lambda (x)
-			      (float (funcall f x)))
+	    (slatec:dqawc (float-integrand-or-lose '$quad_qawc fun var)
 			  (float-or-lose a)
 			  (float-or-lose b)
 			  (float-or-lose c)
@@ -161,7 +175,9 @@
 			  limit lenw 0 iwork work)
 	  (declare (ignore junk z-a z-b z-c z-epsabs z-epsrel z-limit z-lenw last))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qawc) ,fun ,var ,c ,a ,b
 	  ((mequal) $epsrel ,epsrel)
 	  ((mequal) $epsabs ,epsabs)
@@ -176,7 +192,6 @@
 	 (lenw (+ (* 2 leniw) (* 25 maxp1)))
 	 (work (make-array lenw :element-type 'flonum))
 	 (iwork (make-array leniw :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var))
 	 (integr (ecase trig
 		   ((1 %cos $cos) 1)
 		   ((2 %sin $sin) 2))))
@@ -185,8 +200,7 @@
 				   epsabs result abserr neval ier
 				   z-limlst z-lst
 				   z-leniw z-maxp1 z-lenw)
-	    (slatec:dqawf #'(lambda (x)
-			      (float (funcall f x)))
+	    (slatec:dqawf (float-integrand-or-lose '$quad_qawf fun var)
 			  (float-or-lose a)
 			  (float-or-lose omega)
 			  integr
@@ -196,7 +210,9 @@
 	  (declare (ignore junk z-a z-omega z-integr epsabs z-limlst z-lst
 			   z-leniw z-maxp1 z-lenw))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qawf) ,fun ,var ,a ,omega ,trig
 	  ((mequal) $epsabs ,epsabs)
 	  ((mequal) $limit ,limit)
@@ -213,7 +229,6 @@
 	 (lenw (+ (* 2 leniw) (* 25 maxp1)))
 	 (work (make-array lenw :element-type 'flonum))
 	 (iwork (make-array leniw :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var))
 	 (integr (ecase trig
 		   ((1 %cos $cos) 1)
 		   ((2 %sin $sin) 2))))
@@ -221,8 +236,7 @@
 	(multiple-value-bind (junk z-a z-b z-omega z-integr z-epsabs z-epsrel
 				   result abserr neval ier
 				   z-leniw z-maxp1 z-lenw z-lst)
-	    (slatec:dqawo #'(lambda (x)
-			      (float (funcall f x)))
+	    (slatec:dqawo (float-integrand-or-lose '$quad_qawo fun var)
 			  (float-or-lose a)
 			  (float-or-lose b)
 			  (float-or-lose omega)
@@ -234,7 +248,9 @@
 	  (declare (ignore junk z-a z-b z-omega z-integr z-epsabs z-epsrel
 			   z-lst z-leniw z-maxp1 z-lenw))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qawo) ,fun ,var ,a ,b ,omega ,trig
 	  ((mequal) $epsrel ,epsrel)
 	  ((mequal) $epsabs ,epsabs)
@@ -248,14 +264,12 @@
   (quad_argument_check fun var a b) 
   (let* ((lenw (* 4 limit))
 	 (work (make-array lenw :element-type 'flonum))
-	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var)))
+	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
     (handler-case
 	(multiple-value-bind (junk z-a z-b z-alfa z-beta z-int z-epsabs z-epsrel
 				   result abserr neval ier
 				   z-limit z-lenw last)
-	    (slatec:dqaws #'(lambda (x)
-			      (float (funcall f x)))
+	    (slatec:dqaws (float-integrand-or-lose '$quad_qaws fun var)
 			  (float-or-lose a)
 			  (float-or-lose b)
 			  (float-or-lose alfa)
@@ -268,7 +282,9 @@
 	  (declare (ignore junk z-a z-b z-alfa z-beta z-int z-epsabs z-epsrel
 			   z-limit z-lenw last))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qaws) ,fun ,var ,a ,b ,alfa ,beta ,wfun
 	  ((mequal) $epsrel ,epsrel)
 	  ((mequal) $epsabs ,epsabs)
@@ -282,15 +298,13 @@
 	 (leniw (max limit (- (* 3 npts2) 2)))
 	 (lenw (- (* 2 leniw) npts2))
 	 (work (make-array lenw :element-type 'flonum))
-	 (iwork (make-array limit :element-type 'f2cl-lib:integer4))
-	 (f (get-integrand fun var)))
+	 (iwork (make-array limit :element-type 'f2cl-lib:integer4)))
     (map-into p #'float-or-lose (cdr points))
     (handler-case
 	(multiple-value-bind (junk z-a z-b z-npts z-points z-epsabs z-epsrel
 				   result abserr neval ier
 				   z-leniw z-lenw last)
-	    (slatec:dqagp #'(lambda (x)
-			      (float (funcall f x)))
+	    (slatec:dqagp (float-integrand-or-lose '$quad_qagp fun var)
 			  (float-or-lose a)
 			  (float-or-lose b)
 			  npts2
@@ -302,7 +316,9 @@
 	  (declare (ignore junk z-a z-b z-npts z-points z-epsabs z-epsrel
 			   z-leniw z-lenw last))
 	  (list '(mlist) result abserr neval ier))
-      (error ()
+      (error (e)
+	(when *debug-quadpack*
+	  (format t "~S" e))
 	`(($quad_qagp) ,fun ,var ,a ,b ,points
 	  ((mequal) $epsrel ,epsrel)
 	  ((mequal) $epsabs ,epsabs)
@@ -330,24 +346,21 @@
 		    ($control 2)
 		    ($max_message 4)
 		    (otherwise
-		     (merror "Parameter should be current_error, control, or max_mmessage")))
+		     (merror "Parameter should be current_error, control, or max_message")))
 		  (or new-value 0)
 		  (if new-value t nil))))
 
 (defun $quad_control (parameter &rest new-value)
   (quad-control parameter (if new-value (car new-value))))
 
+
 (macrolet
     ((frob (mname iname args valid-keys)
-       (let* ((keylist (gensym "KEY-LIST-"))
-	      (options (gensym "OPTIONS-")))
-	 `(defun ,mname (,@args &rest ,options)
-	    (let
-		((,keylist (lispify-maxima-keyword-options ,options ',valid-keys))
-		 ;; BIND EVIL SPECIAL VARIABLE *PLOT-REALPART* HERE ...
-		 (*plot-realpart* nil))
-	      (declare (special *plot-realpart*))
-	      (apply ',iname ,@args ,keylist))))))
+       `(defun-checked ,mname ((,@args) ,@valid-keys)
+	  ;; BIND EVIL SPECIAL VARIABLE *PLOT-REALPART* HERE ...
+	  (let ((*plot-realpart* nil))
+	    (declare (special *plot-realpart*))
+	    (apply ',iname ,@args keylist)))))
   (frob $quad_qag quad-qag (fun var a b key) ($epsrel $limit $epsabs))
   (frob $quad_qags quad-qags (fun var a b) ($epsrel $limit $epsabs))
   (frob $quad_qagi quad-qagi (fun var a b) ($epsrel $limit $epsabs))
@@ -356,7 +369,7 @@
   (frob $quad_qawo quad-qawo (fun var a b omega trig) ($epsrel $limit $epsabs $maxp1))
   (frob $quad_qaws quad-qaws (fun var a b alfa beta wfun) ($epsrel $limit $epsabs))
   (frob $quad_qagp quad-qagp (fun var a b points) ($epsrel $limit $epsabs)))
-  
+
 ;; Tests
 ;;
 ;; These tests were taken from the QUADPACK book.
