@@ -1,6 +1,6 @@
 ;;; -*-  Mode: Lisp; Package: Maxima; Syntax: Common-Lisp; Base: 10 -*- ;;;;
 ;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
-;;;     The data in this file contains enhancments.                    ;;;;;
+;;;     The data in this file contains enhancements.                   ;;;;;
 ;;;                                                                    ;;;;;
 ;;;  Copyright (c) 1984,1987 by William Schelter,University of Texas   ;;;;;
 ;;;     All rights reserved                                            ;;;;;
@@ -26,7 +26,7 @@
 		       $rules $props $ratvars
 		       varlist genvar
 		       $gensumnum checkfactors $features featurel
-		       $weightlevels tellratlist $dontfactor
+		       tellratlist $dontfactor
 		       dispflag savefile $%% $error
 		       opers *ratweights $ratweights
 		       $stringdisp $lispdisp
@@ -209,6 +209,7 @@
   ;; Should really get the truename of FILE.
   (if printp (format t (intl:gettext "loadfile: loading ~A.~%") file))
   (let* ((path (pathname file))
+	 (*package* (find-package :maxima))
 	 ($load_pathname path)
 	 (*read-base* 10.)
 	 (tem (errset #-sbcl (load (pathname file)) #+sbcl (with-compilation-unit nil (load (pathname file))))))
@@ -331,22 +332,30 @@
 	      (mapc #'kill1 (cdr $infolists))
 	      (setq $ratvars '((mlist simp)) varlist nil genvar nil
 		    checkfactors nil greatorder nil lessorder nil $gensumnum 0
-		    $weightlevels '((mlist)) *ratweights nil $ratweights
+		    *ratweights nil $ratweights
 		    '((mlist simp))
 		    tellratlist nil $dontfactor '((mlist)) $setcheck nil)
 	      (killallcontexts))
 	     ((setq z (assoc x '(($inlabels . $inchar) ($outlabels . $outchar) ($linelabels . $linechar)) :test #'eq))
 	      (mapc #'(lambda (y) (remvalue y '$kill))
 		    (getlabels* (eval (cdr z)) nil)))
-	     ((and (eq (ml-typep x) 'fixnum) (not (< x 0))) (remlabels x))
+	     ((and (fixnump x) (>= x 0)) (remlabels x))
 	     ((atom x) (kill1-atom x))
-	     ((and (eq (caar x) 'mlist) (eq (ml-typep (cadr x)) 'fixnum)
+	     ((and (eq (caar x) 'mlist) (fixnump (cadr x))
 		   (or (and (null (cddr x))
-			    (setq x (append x (ncons (cadr x)))))
-		       (and (eq (ml-typep (caddr x)) 'fixnum)
-			    (not (> (cadr x) (caddr x))))))
+                         (setq x (append x (ncons (cadr x)))))
+                      (and (fixnump (caddr x))
+                         (not (> (cadr x) (caddr x))))))
 	      (let (($linenum (caddr x))) (remlabels (- (caddr x) (cadr x)))))
 	     ((setq z (mgetl (caar x) '(hashar array))) (remarrelem z x))
+	     ((and ($subvarp x)
+		   (boundp (caar x))
+		   (hash-table-p (setq z (symbol-value (caar x)))))
+	      ; Evaluate the subscripts (as is done in ARRSTORE)
+	      (let ((indices (mevalargs (cdr x))))
+		(if (gethash 'dim1 z)
+		  (remhash (car indices) z)
+		  (remhash indices z))))
          ((eq (caar x) '$@) (mrecord-kill x))
 	     ((and (eq (caar x) '$allbut)
 		   (not (dolist (u (cdr x))
@@ -472,7 +481,7 @@
        (setq inchar (getlabcharn $inchar)) ; Only the 1st alphabetic char. of $INCHAR is tested
        (setq timep $showtime grindp $grind)
        (do ((x x (cdr x)))( (null x))
-	 (cond ((eq (ml-typep (car x)) 'fixnum) (setq numbp (car x)))
+	 (cond ((fixnump (car x)) (setq numbp (car x)))
 	       ((eq (car x) '$all))
 	       ((eq (car x) '$slow) (setq slowp t))
 	       ((eq (car x) '$nostring) (setq nostringp t))
@@ -517,10 +526,10 @@
 
 (defun listargp (x)
   (let (high)
-    (if (and ($listp x) (eq (ml-typep (cadr x)) 'fixnum)
-	     (or (and (null (cddr x)) (setq high (cadr x)))
-		 (and (eq (ml-typep (setq high (caddr x))) 'fixnum)
-		      (not (> (cadr x) high)))))
+    (if (and ($listp x) (fixnump (cadr x))
+           (or (and (null (cddr x)) (setq high (cadr x)))
+              (and (fixnump (setq high (caddr x)))
+                 (not (> (cadr x) high)))))
 	(cons (cadr x) high))))
 
 (defmspec $alias (form)
@@ -535,9 +544,6 @@
   (cond ((nonsymchk x '$alias))
 	((nonsymchk y '$alias))
         ((eq x y) y) ; x is already the alias of y
-; Not needed. We return the alias immediately if we already have one.
-;	((not (eq (getcharn x 1) #\$))
-;	 (merror "-ed symbols may not be aliased. ~M" x))
 	((get x 'reversealias)
 	 (if (not (eq x y))
 	     (merror (intl:gettext "alias: ~M already has an alias.") x)))
@@ -564,8 +570,8 @@
 	((numberp x) x)
 	((null x) 'false)
 	((eq x t) 'true)
-    ((member (getcharn x 1) '(#\$ #\%))
-     (intern (subseq (string x) 1)))
+        ((member (get-first-char x) '(#\$ #\%) :test #'char=)
+         (intern (subseq (string x) 1)))
 	(t x)))
 
 (defmfun fullstrip (x)
@@ -657,7 +663,8 @@
   (cond
     ((symbolp name) name)
     ((stringp name)
-     (getalias (or (getopr0 name) (implode (cons #\$ (coerce name 'list))))))))
+     (getalias (or (getopr0 name) (implode (cons #\$ (coerce name 'list))))))
+    (t name)))
 
 (defmspec $stringout (x)
   (setq x (cdr x))
@@ -718,7 +725,7 @@
 
 (defmfun $%th (x)
   (prog (l outchar)
-     (if (or (not (eq (ml-typep x) 'fixnum)) (= x 0))
+     (if (or (not (fixnump x)) (zerop x))
 	 (improper-arg-err x '$%th))
      (if (> x 0) (setq x (- x)))
      (if (cdr $labels)
@@ -762,14 +769,14 @@
 (defmspec $errcatch (form)
   (let ((errcatch (cons bindlist loclist)) ret)
     (if (null (setq ret (let (*mdebug*)
-			  (errset (mevaln (cdr form)) lisperrprint))))
+			  (errset (rat-error-to-merror (mevaln (cdr form))) lisperrprint))))
 	(errlfun1 errcatch))
     (cons '(mlist) ret)))
 
 (defmspec $catch (form)
   (let ((mcatch (cons bindlist loclist)))
     (prog1
-	(catch 'mcatch (mevaln (cdr form)))
+	(catch 'mcatch (rat-error-to-merror (mevaln (cdr form))))
       (errlfun1 mcatch))))
 
 (defmfun $throw (exp)
@@ -802,7 +809,8 @@
 ;; File-processing stuff.
 
 (defmfun mterpri ()
-   (terpri))
+   (terpri)
+   (force-output))
 
 (defmspec $status (form)
   (setq form (cdr form))
@@ -848,7 +856,6 @@
 
 (mapc #'(lambda (x) (putprop (car x) (cadr x) 'assign))
       '(($debugmode debugmode1)
-	($ttyintfun ttyintfunsetup)
 	($fpprec fpprec1) ($poislim poislim1)
 	($default_let_rule_package let-rule-setter)
 	($current_let_rule_package let-rule-setter)
